// ============================================================================
// Copyright (c) 2015 by Terasic Technologies Inc.
// ============================================================================
//
// Permission:
//
//   Terasic grants permission to use and modify this code for use
//   in synthesis for all Terasic Development Boards and Altera Development 
//   Kits made by Terasic.  Other use of this code, including the selling 
//   ,duplication, or modification of any portion is strictly prohibited.
//
// Disclaimer:
//
//   This VHDL/Verilog or C/C++ source code is intended as a design reference
//   which illustrates how these types of functions can be implemented.
//   It is the user's responsibility to verify their design for
//   consistency and functionality through the use of formal
//   verification methods.  Terasic provides no warranty regarding the use 
//   or functionality of this code.
//
// ============================================================================
//           
//  Terasic Technologies Inc
//  9F., No.176, Sec.2, Gongdao 5th Rd, East Dist, Hsinchu City, 30070. Taiwan
//  
//  
//                     web: http://www.terasic.com/  
//                     email: support@terasic.com
//
// ============================================================================
//Date:  Thu Sep 24 10:51:40 2015
// ============================================================================

`define ENABLE_DDR3
`define ENABLE_PCIE
//`define ENABLE_SATA
//`define ENABLE_FMCA_XCVR
//`define ENABLE_FMCB_XCVR
//`define ENABLE_FMCC_XCVR
//`define ENABLE_FMCD_XCVR
`define DDR3_RANK_NUM   1  //2

module PCIE_DDR3(

      ///////// CLOCK /////////
      input              OSC_50_B3B,
      input              OSC_50_B4A,
      input              OSC_50_B4D,
      input              OSC_50_B7A,
      input              OSC_50_B7D,
      input              OSC_50_B8A,
      input              OSC_50_B8D,

      ///////// KEY /////////
      input              CPU_RESET_n,
      input    [ 3: 0]   BUTTON, //KEY is Low-Active

      ///////// SW /////////
      input    [ 3: 0]   SW,

      ///////// LED /////////
      output   [ 3: 0]   LED, //LED is Low-Active

      ///////// FAN /////////
      input              FAN_ALERT_n,

      ///////// SSRAM /////////
      output             SSRAM_CLK,
      output             SSRAM_CKE_n,
      output             SSRAM_CE_n,
      output             SSRAM_WE_n,
      output             SSRAM_OE_n,
      output             SSRAM_ADV,
      output             SSRAM_BWA_n,
      output             SSRAM_BWB_n,

      ///////// FLASH /////////
      output             FLASH_CLK,
      output             FLASH_CE_n,
      output             FLASH_WE_n,
      output             FLASH_OE_n,
      output             FLASH_ADV_n,
      output             FLASH_RESET_n,
      input              FLASH_RDY_BSY_n,

      ///////// FSM /////////
      output   [26: 1]   FSM_A,
      inout    [15: 0]   FSM_D,

      ///////// SD Card /////////
      output             SD_CLK,
      inout    [ 3: 0]   SD_DATA,
      inout              SD_CMD,

`ifdef ENABLE_DDR3
      ///////// DDR3 /////////
      input              DDR3_REFCLK_p,
      output   [15: 0]   DDR3_A,
      output   [ 2: 0]   DDR3_BA,
      output   [ (`DDR3_RANK_NUM-1):0]   DDR3_CK,
      output   [ (`DDR3_RANK_NUM-1):0]   DDR3_CK_n,
      output   [ (`DDR3_RANK_NUM-1):0]   DDR3_CKE,
      inout    [ 7: 0]   DDR3_DQS,
      inout    [ 7: 0]   DDR3_DQS_n,
      inout    [63: 0]   DDR3_DQ,
      output   [ 7: 0]   DDR3_DM,
      output   [ (`DDR3_RANK_NUM-1):0]   DDR3_CS_n,
      output             DDR3_WE_n,
      output             DDR3_CAS_n,
      output             DDR3_RAS_n,
      output             DDR3_RESET_n,
      output   [ (`DDR3_RANK_NUM-1):0]   DDR3_ODT,
      input              DDR3_EVENT_n,
      output             DDR3_SCL,
      inout              DDR3_SDA,
`endif /*ENABLE_DDR3*/

      ///////// RZQ /////////
      input              RZQ_DDR3,
      input              RZQ_FMC,

      ///////// Uart to USB /////////
      output             UART_TX,
      input              UART_RX,

      ///////// TPS40422 /////////
      output             TPS40422_CLK,
      inout              TPS40422_DATA,
      input              TPS40422_ALERT,

      ///////// External PLL /////////
      output             LMK04906_CLK,
      output             LMK04906_DATAIN,
      input              LMK04906_DATAOUT,
      output             LMK04906_LE,

      ///////// I2C /////////
      output             CLOCK_SCL,
      inout              CLOCK_SDA,

      ///////// Shared I2C /////////
      inout              FPGA_I2C_SCL,
      inout              FPGA_I2C_SDA,

      ///////// Temperature /////////
      input              TEMP_INT_n,
      input              TEMP_OVERT_n,

      ///////// POWER Monitor /////////
      input              POWER_MONITOR_ALERT,

      ///////// GPIO /////////
      inout    [35: 0]   GPIO,

      ///////// FMCA /////////
      input    [ 1: 0]   FMCA_CLK_M2C_p,
      input    [ 1: 0]   FMCA_CLK_M2C_n,
      input              FMCA_HA_RX_CLK_p,
      input              FMCA_HA_RX_CLK_n,
      output             FMCA_HA_TX_CLK_p,
      output             FMCA_HA_TX_CLK_n,
      input              FMCA_HB_RX_CLK_p,
      input              FMCA_HB_RX_CLK_n,
      output             FMCA_HB_TX_CLK_p,
      output             FMCA_HB_TX_CLK_n,
      input              FMCA_LA_RX_CLK_p,
      input              FMCA_LA_RX_CLK_n,
      output             FMCA_LA_TX_CLK_p,
      output             FMCA_LA_TX_CLK_n,
      inout    [10: 0]   FMCA_HA_TX_p,
      inout    [10: 0]   FMCA_HA_TX_n,
      inout    [10: 0]   FMCA_HA_RX_p,
      inout    [10: 0]   FMCA_HA_RX_n,
      inout    [10: 0]   FMCA_HB_TX_p,
      inout    [10: 0]   FMCA_HB_TX_n,
      inout    [10: 0]   FMCA_HB_RX_p,
      inout    [10: 0]   FMCA_HB_RX_n,
      inout    [16: 0]   FMCA_LA_TX_p,
      inout    [16: 0]   FMCA_LA_TX_n,
      inout    [14: 0]   FMCA_LA_RX_p,
      inout    [14: 0]   FMCA_LA_RX_n,

`ifdef ENABLE_FMCA_XCVR	
      input    [ 1: 0]   FMCA_GBTCLK_M2C_p,
      input    [ 1: 0]   FMCA_ONBOARD_REFCLK_p,
      output   [ 9: 0]   FMCA_DP_C2M_p,
      input    [ 9: 0]   FMCA_DP_M2C_p,
`endif /*ENABLE_FMCA_XCVR*/

      output   [ 1: 0]   FMCA_GA,
      inout              FMCA_SCL,
      inout              FMCA_SDA,

      ///////// FMCB /////////
      input    [ 1: 0]   FMCB_CLK_M2C_p,
      input    [ 1: 0]   FMCB_CLK_M2C_n,
      input              FMCB_LA_RX_CLK_p,
      input              FMCB_LA_RX_CLK_n,
      output             FMCB_LA_TX_CLK_p,
      output             FMCB_LA_TX_CLK_n,
      inout    [16: 0]   FMCB_LA_TX_p,
      inout    [16: 0]   FMCB_LA_TX_n,
      inout    [14: 0]   FMCB_LA_RX_p,
      inout    [14: 0]   FMCB_LA_RX_n,

`ifdef ENABLE_FMCB_XCVR	
      input    [ 0: 0]   FMCB_GBTCLK_M2C_p,
      input    [ 0: 0]   FMCB_ONBOARD_REFCLK_p,
      output   [ 0: 0]   FMCB_DP_C2M_p,
      input    [ 0: 0]   FMCB_DP_M2C_p,
`endif /*ENABLE_FMCB_XCVR*/

      output   [ 1: 0]   FMCB_GA,
      inout              FMCB_SCL,
      inout              FMCB_SDA,

      ///////// FMCC /////////
      input    [ 1: 0]   FMCC_CLK_M2C_p,
      input    [ 1: 0]   FMCC_CLK_M2C_n,
      input              FMCC_LA_RX_CLK_p,
      input              FMCC_LA_RX_CLK_n,
      output             FMCC_LA_TX_CLK_p,
      output             FMCC_LA_TX_CLK_n,
      inout    [16: 0]   FMCC_LA_TX_p,
      inout    [16: 0]   FMCC_LA_TX_n,
      inout    [14: 0]   FMCC_LA_RX_p,
      inout    [14: 0]   FMCC_LA_RX_n,

`ifdef ENABLE_FMCC_XCVR	
      input    [ 0: 0]   FMCC_GBTCLK_M2C_p,
      input    [ 0: 0]   FMCC_ONBOARD_REFCLK_p,
      output   [ 0: 0]   FMCC_DP_C2M_p,
      input    [ 0: 0]   FMCC_DP_M2C_p,
`endif /*ENABLE_FMCC_XCVR*/

      output   [ 1: 0]   FMCC_GA,
      inout              FMCC_SCL,
      inout              FMCC_SDA,

      ///////// FMCD /////////
      input    [ 1: 0]   FMCD_CLK_M2C_p,
      input    [ 1: 0]   FMCD_CLK_M2C_n,
      input              FMCD_HA_RX_CLK_p,
      input              FMCD_HA_RX_CLK_n,
      output             FMCD_HA_TX_CLK_p,
      output             FMCD_HA_TX_CLK_n,
      input              FMCD_HB_RX_CLK_p,
      input              FMCD_HB_RX_CLK_n,
      output             FMCD_HB_TX_CLK_p,
      output             FMCD_HB_TX_CLK_n,
      input              FMCD_LA_RX_CLK_p,
      input              FMCD_LA_RX_CLK_n,
      output             FMCD_LA_TX_CLK_p,
      output             FMCD_LA_TX_CLK_n,
      inout    [10: 0]   FMCD_HA_TX_p,
      inout    [10: 0]   FMCD_HA_TX_n,
      inout    [10: 0]   FMCD_HA_RX_p,
      inout    [10: 0]   FMCD_HA_RX_n,
      inout    [10: 0]   FMCD_HB_TX_p,
      inout    [10: 0]   FMCD_HB_TX_n,
      inout    [10: 0]   FMCD_HB_RX_p,
      inout    [10: 0]   FMCD_HB_RX_n,
      inout    [16: 0]   FMCD_LA_TX_p,
      inout    [16: 0]   FMCD_LA_TX_n,
      inout    [14: 0]   FMCD_LA_RX_p,
      inout    [14: 0]   FMCD_LA_RX_n,
		
`ifdef  ENABLE_FMCD_XCVR
      input    [ 1: 0]   FMCD_GBTCLK_M2C_p,
      input    [ 1: 0]   FMCD_ONBOARD_REFCLK_p,
      output   [ 9: 0]   FMCD_DP_C2M_p,
      input    [ 9: 0]   FMCD_DP_M2C_p,
`endif /*ENABLE_FMCD_XCVR*/	

      output   [ 1: 0]   FMCD_GA,
      inout              FMCD_SCL,
      inout              FMCD_SDA,


`ifdef ENABLE_PCIE
      ///////// PCIE /////////
      //input              PCIE_ONBOARD_REFCLK_p,
      input              PCIE_REFCLK_p,
      output   [ 3: 0]   PCIE_TX_p,
      input    [ 3: 0]   PCIE_RX_p,
      input              PCIE_PERST_n,
      output             PCIE_WAKE_n,
`endif /*ENABLE_PCIE*/

`ifdef ENABLE_SATA
      ///////// SATA /////////
      input              SATA_HOST_REFCLK_p,
      output             SATA_HOST_TX_p,
      input              SATA_HOST_RX_p,
      input              SATA_DEVICE_REFCLK_p,
      output             SATA_DEVICE_TX_p,
      input              SATA_DEVICE_RX_p, 
`endif /*ENABLE_SATA*/


      ///////// SMA /////////
      input              SMA_CLKIN_p,
      output             SMA_CLKOUT_p

);

//=======================================================
//  REG/WIRE declarations
//=======================================================
wire [3:0]	pio_led;
wire [3:0]	pio_button;
wire [31:0] pcie_hip_ctrl_test_in;
wire 			ddr3_local_init_done;
wire 			ddr3_local_cal_success;
wire 			ddr3_local_cal_fail;

//////////////////////
// PCIE RESET
wire     any_rstn;
reg      any_rstn_r  /* synthesis ALTERA_ATTRIBUTE = "SUPPRESS_DA_RULE_INTERNAL=R102"  */;
reg      any_rstn_rr /* synthesis ALTERA_ATTRIBUTE = "SUPPRESS_DA_RULE_INTERNAL=R102"  */;
//=======================================================
//  Structural coding
//=======================================================
assign 	pcie_hip_ctrl_test_in[4:0]  =  5'b01000;
assign 	pcie_hip_ctrl_test_in[5] =  1'b1;
assign 	pcie_hip_ctrl_test_in[31:6] =  26'h2;

assign 	PCIE_WAKE_n = 1'b1;
assign 	any_rstn = PCIE_PERST_n & CPU_RESET_n ;
  
//reset Synchronizer
always @(posedge OSC_50_B4A or negedge any_rstn)
    begin
      if (any_rstn == 0)
        begin
          any_rstn_r <= 0;
          any_rstn_rr <= 0;
        end
      else
        begin
          any_rstn_r <= 1;
          any_rstn_rr <= any_rstn_r;
        end
    end


top u0 (
        .clk_clk                                                 (OSC_50_B4A),                                                 	// clk.clk
        .hip_ctrl_test_in                                        (pcie_hip_ctrl_test_in),                                        // hip_ctrl.test_in
        .hip_ctrl_simu_mode_pipe                                 (1'b0),                                 								//         .simu_mode_pipe
        .hip_pipe_sim_pipe_pclk_in                               (1'b0),                              								 	// hip_pipe.sim_pipe_pclk_in
        .hip_pipe_sim_pipe_rate                                  (1'b0),                                  								//         .sim_pipe_rate
        .hip_serial_rx_in0                                       (PCIE_RX_p[0]),                                       				// hip_serial.rx_in0
        .hip_serial_rx_in1                                       (PCIE_RX_p[1]),                                       				//           .rx_in1
        .hip_serial_rx_in2                                       (PCIE_RX_p[2]),                                       				//           .rx_in2
        .hip_serial_rx_in3                                       (PCIE_RX_p[3]),                                       				//           .rx_in3
        .hip_serial_tx_out0                                      (PCIE_TX_p[0]),                                      				//           .tx_out0
        .hip_serial_tx_out1                                      (PCIE_TX_p[1]),                                      				//           .tx_out1
        .hip_serial_tx_out2                                      (PCIE_TX_p[2]),                                      				//           .tx_out2
        .hip_serial_tx_out3                                      (PCIE_TX_p[3]),                                      				//           .tx_out3
        .pcie_rstn_npor                                          (any_rstn_rr),                                          			// pcie_rstn.npor
        .pcie_rstn_pin_perst                                     (PCIE_PERST_n),                                     				//          .pin_perst
        .refclk_clk                                              (PCIE_REFCLK_p),                                              	// refclk.clk
        .reset_reset_n                                           ((CPU_RESET_n==1'b0)?1'b0:(PCIE_PERST_n==1'b0)?1'b0:1'b1),      // reset.reset_n
        .pio_led_external_connection_export                      (pio_led),                      											// pio_led_external_connection.export
        .pio_button_external_connection_export                   (pio_button),                    											// pio_button_external_connection.export

		  .mem_if_ddr3_emif_0_pll_ref_clk_clk                      (OSC_50_B3B),                      			 							// mem_if_ddr3_emif_0_pll_ref_clk.clk		  
        .memory_mem_a                              				  (DDR3_A),                              									// memory.mem_a
        .memory_mem_ba                            					  (DDR3_BA),                             									//       .mem_ba
        .memory_mem_ck                           					  (DDR3_CK),                             									//       .mem_ck
        .memory_mem_ck_n                         					  (DDR3_CK_n),                           									//       .mem_ck_n
        .memory_mem_cke                         					  (DDR3_CKE),                            									//       .mem_cke
        .memory_mem_cs_n                         					  (DDR3_CS_n),                           									//       .mem_cs_n
        .memory_mem_dm                           					  (DDR3_DM),                             									//       .mem_dm
        .memory_mem_ras_n                         					  (DDR3_RAS_n),                          									//       .mem_ras_n
        .memory_mem_cas_n                        					  (DDR3_CAS_n),                          									//       .mem_cas_n
        .memory_mem_we_n                        					  (DDR3_WE_n),                           									//       .mem_we_n
        .memory_mem_reset_n                      					  (DDR3_RESET_n),                       									//       .mem_reset_n
        .memory_mem_dq                          					  (DDR3_DQ),                             									//       .mem_dq
        .memory_mem_dqs                          					  (DDR3_DQS),                            									//       .mem_dqs
        .memory_mem_dqs_n                        					  (DDR3_DQS_n),                          									//       .mem_dqs_n
        .memory_mem_odt                          					  (DDR3_ODT),                            									//       .mem_odt
        .oct_rzqin                                					  (RZQ_DDR3),                                 							// oct.rzqin
        .mem_if_ddr3_emif_0_status_local_init_done   				  (ddr3_local_init_done),   													// mem_if_ddr3_emif_status.local_init_done
        .mem_if_ddr3_emif_0_status_local_cal_success				  (ddr3_local_cal_success), 													//                        .local_cal_success
        .mem_if_ddr3_emif_0_status_local_cal_fail  			     (ddr3_local_cal_fail),    													//                        .local_cal_fail
        .ddr3_status_external_connection_export    				  ({ddr3_local_cal_success, ddr3_local_cal_fail, ddr3_local_init_done})// ddr3_status_external_connection.export

    );

assign 	LED = ~pio_led; // led low-active
assign 	pio_button = ~BUTTON; // button low-active
	

endmodule
